@@ -567,7 +567,11 @@ albumart_request_download (TrackerStorage *storage,
 		                        ALBUMARTER_PATH,
 		                        ALBUMARTER_INTERFACE,
 		                        "Queue",
-		                        g_variant_new ("(sssu)", artist, album, "album", 0),
+		                        g_variant_new ("(sssu)",
+		                                       artist ? artist : "",
+		                                       album ? album : "",
+		                                       "album",
+		                                       0),
 		                        NULL,
 		                        G_DBUS_CALL_FLAGS_NONE,
 		                        -1,
@@ -698,6 +702,8 @@ tracker_albumart_init (void)
 
 	g_return_val_if_fail (initialized == FALSE, FALSE);
 
+	tracker_albumart_plugin_init ();
+
 	albumart_storage = tracker_storage_new ();
 
 	/* Cache to know if we have already handled uris */
@@ -738,6 +744,8 @@ tracker_albumart_shutdown (void)
 		g_object_unref (albumart_storage);
 	}
 
+	tracker_albumart_plugin_shutdown ();
+
 	initialized = FALSE;
 }
 
