@@ -2,18 +2,18 @@
  * Copyright (C) 2006, Jamie McCracken <jamiemcc@gnome.org>
  * Copyright (C) 2008, Nokia <ivan.frade@nokia.com>
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 2.1 of the License, or (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public
- * License along with this program; if not, write to the
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the
  * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
  * Boston, MA  02110-1301, USA.
  */
@@ -27,6 +27,7 @@
 #include <png.h>
 
 #include <libtracker-common/tracker-file-utils.h>
+#include <libtracker-common/tracker-date-time.h>
 #include <libtracker-extract/tracker-extract.h>
 
 #define RFC1123_DATE_FORMAT "%d %B %Y %H:%M:%S %z"
@@ -92,93 +93,101 @@ read_metadata (TrackerSparqlBuilder *preupdate,
                TrackerSparqlBuilder *metadata,
                png_structp           png_ptr,
                png_infop             info_ptr,
+               png_infop             end_ptr,
                const gchar          *uri)
 {
 	MergeData md = { 0 };
 	PngData pd = { 0 };
 	TrackerExifData *ed = NULL;
 	TrackerXmpData *xd = NULL;
+	png_infop info_ptrs[2];
 	png_textp text_ptr;
+	gint info_index;
 	gint num_text;
 	gint i;
 	gint found;
 	GPtrArray *keywords;
 
-	if ((found = png_get_text (png_ptr, info_ptr, &text_ptr, &num_text)) < 1) {
-		g_debug ("Calling png_get_text() returned %d (< 1)", found);
-		return;
-	}
+	info_ptrs[0] = info_ptr;
+	info_ptrs[1] = end_ptr;
 
-	for (i = 0; i < num_text; i++) {
-		if (!text_ptr[i].key || !text_ptr[i].text || text_ptr[i].text[0] == '\0') {
+	for (info_index = 0; info_index < 2; info_index++) {
+		if ((found = png_get_text (png_ptr, info_ptrs[info_index], &text_ptr, &num_text)) < 1) {
+			g_debug ("Calling png_get_text() returned %d (< 1)", found);
 			continue;
 		}
+
+		for (i = 0; i < num_text; i++) {
+			if (!text_ptr[i].key || !text_ptr[i].text || text_ptr[i].text[0] == '\0') {
+				continue;
+			}
 		
-#if defined(HAVE_EXEMPI) && defined(PNG_iTXt_SUPPORTED)
-		if (g_strcmp0 ("XML:com.adobe.xmp", text_ptr[i].key) == 0) {
-			/* ATM tracker_extract_xmp_read supports setting xd
-			 * multiple times, keep it that way as here it's
-			 * theoretically possible that the function gets
-			 * called multiple times 
+	#if defined(HAVE_EXEMPI) && defined(PNG_iTXt_SUPPORTED)
+			if (g_strcmp0 ("XML:com.adobe.xmp", text_ptr[i].key) == 0) {
+				/* ATM tracker_extract_xmp_read supports setting xd
+				 * multiple times, keep it that way as here it's
+				 * theoretically possible that the function gets
+				 * called multiple times 
+				 */
+				xd = tracker_xmp_new (text_ptr[i].text,
+					              text_ptr[i].itxt_length,
+					              uri);
+				continue;
+			}
+	#endif
+
+	#if defined(HAVE_LIBEXIF) && defined(PNG_iTXt_SUPPORTED)
+			/* I'm not certain this is the key for EXIF. Using key according to
+			 * this document about exiftool:
+			 * http://www.sno.phy.queensu.ca/~phil/exiftool/TagNames/PNG.html#TextualData 
 			 */
-			xd = tracker_xmp_new (text_ptr[i].text,
-			                      text_ptr[i].itxt_length,
-			                      uri);
-			continue;
-		}
-#endif
+			if (g_strcmp0 ("Raw profile type exif", text_ptr[i].key) == 0) {
+				ed = tracker_exif_new (text_ptr[i].text,
+					               text_ptr[i].itxt_length,
+					               uri);
+				continue;
+			}
+	#endif /* HAVE_LIBEXIF */
 
-#if defined(HAVE_LIBEXIF) && defined(PNG_iTXt_SUPPORTED)
-		/* I'm not certain this is the key for EXIF. Using key according to
-		 * this document about exiftool:
-		 * http://www.sno.phy.queensu.ca/~phil/exiftool/TagNames/PNG.html#TextualData 
-		 */
-		if (g_strcmp0 ("Raw profile type exif", text_ptr[i].key) == 0) {
-			ed = tracker_exif_new (text_ptr[i].text,
-			                       text_ptr[i].itxt_length,
-			                       uri);
-			continue;
-		}
-#endif /* HAVE_LIBEXIF */
+			if (g_strcmp0 (text_ptr[i].key, "Author") == 0) {
+				pd.author = text_ptr[i].text;
+				continue;
+			}
 
-		if (g_strcmp0 (text_ptr[i].key, "Author") == 0) {
-			pd.author = text_ptr[i].text;
-			continue;
-		}
+			if (g_strcmp0 (text_ptr[i].key, "Creator") == 0) {
+				pd.creator = text_ptr[i].text;
+				continue;
+			}
 
-		if (g_strcmp0 (text_ptr[i].key, "Creator") == 0) {
-			pd.creator = text_ptr[i].text;
-			continue;
-		}
+			if (g_strcmp0 (text_ptr[i].key, "Description") == 0) {
+				pd.description = text_ptr[i].text;
+				continue;
+			}
 
-		if (g_strcmp0 (text_ptr[i].key, "Description") == 0) {
-			pd.description = text_ptr[i].text;
-			continue;
-		}
+			if (g_strcmp0 (text_ptr[i].key, "Comment") == 0) {
+				pd.comment = text_ptr[i].text;
+				continue;
+			}
 
-		if (g_strcmp0 (text_ptr[i].key, "Comment") == 0) {
-			pd.comment = text_ptr[i].text;
-			continue;
-		}
+			if (g_strcmp0 (text_ptr[i].key, "Copyright") == 0) {
+				pd.copyright = text_ptr[i].text;
+				continue;
+			}
 
-		if (g_strcmp0 (text_ptr[i].key, "Copyright") == 0) {
-			pd.copyright = text_ptr[i].text;
-			continue;
-		}
+			if (g_strcmp0 (text_ptr[i].key, "Creation Time") == 0) {
+				pd.creation_time = rfc1123_to_iso8601_date (text_ptr[i].text);
+				continue;
+			}
 
-		if (g_strcmp0 (text_ptr[i].key, "Creation Time") == 0) {
-			pd.creation_time = rfc1123_to_iso8601_date (text_ptr[i].text);
-			continue;
-		}
+			if (g_strcmp0 (text_ptr[i].key, "Title") == 0) {
+				pd.title = text_ptr[i].text;
+				continue;
+			}
 
-		if (g_strcmp0 (text_ptr[i].key, "Title") == 0) {
-			pd.title = text_ptr[i].text;
-			continue;
-		}
-
-		if (g_strcmp0 (text_ptr[i].key, "Disclaimer") == 0) {
-			pd.disclaimer = text_ptr[i].text;
-			continue;
+			if (g_strcmp0 (text_ptr[i].key, "Disclaimer") == 0) {
+				pd.disclaimer = text_ptr[i].text;
+				continue;
+			}
 		}
 	}
 
@@ -240,10 +249,10 @@ read_metadata (TrackerSparqlBuilder *preupdate,
 		g_free (uri);
 	}
 
-	if (md.date) {
-		tracker_sparql_builder_predicate (metadata, "nie:contentCreated");
-		tracker_sparql_builder_object_unvalidated (metadata, md.date);
-	}
+	tracker_guarantee_date_from_file_mtime (metadata,
+	                                        "nie:contentCreated",
+	                                        md.date,
+	                                        uri);
 
 	if (md.description) {
 		tracker_sparql_builder_predicate (metadata, "nie:description");
@@ -255,10 +264,10 @@ read_metadata (TrackerSparqlBuilder *preupdate,
 		tracker_sparql_builder_object_unvalidated (metadata, md.copyright);
 	}
 
-	if (md.title) {
-		tracker_sparql_builder_predicate (metadata, "nie:title");
-		tracker_sparql_builder_object_unvalidated (metadata, md.title);
-	}
+	tracker_guarantee_title_from_file (metadata,
+	                                   "nie:title",
+	                                   md.title,
+	                                   uri);
 
 	if (md.make || md.model) {
 		gchar *equip_uri;
@@ -414,35 +423,15 @@ read_metadata (TrackerSparqlBuilder *preupdate,
 	if (xd->address || xd->country || xd->city) {
 		gchar *addruri;
 
-		tracker_sparql_builder_predicate (metadata, "mlo:location");
+		tracker_sparql_builder_predicate (metadata, "slo:location");
 
 		tracker_sparql_builder_object_blank_open (metadata);
 		tracker_sparql_builder_predicate (metadata, "a");
-		tracker_sparql_builder_object (metadata, "mlo:GeoPoint"); /* GeoPoint */
-
-		if (xd->address) {
-			tracker_sparql_builder_predicate (metadata, "mlo:address");
-			tracker_sparql_builder_object_unvalidated (metadata, xd->address);
-		}
-
-		if (xd->state) {
-			tracker_sparql_builder_predicate (metadata, "mlo:state");
-			tracker_sparql_builder_object_unvalidated (metadata, xd->state);
-		}
-
-		if (xd->city) {
-			tracker_sparql_builder_predicate (metadata, "mlo:city");
-			tracker_sparql_builder_object_unvalidated (metadata, xd->city);
-		}
-
-		if (xd->country) {
-			tracker_sparql_builder_predicate (metadata, "mlo:country");
-			tracker_sparql_builder_object_unvalidated (metadata, xd->country);
-		}
+		tracker_sparql_builder_object (metadata, "slo:GeoPoint"); /* GeoPoint */
 
 		addruri = tracker_sparql_get_uuid_urn ();
 
-		tracker_sparql_builder_predicate (metadata, "mlo:asPostalAddress");
+		tracker_sparql_builder_predicate (metadata, "slo:postalAddress");
 		tracker_sparql_builder_object_iri (metadata, addruri);
 
 		tracker_sparql_builder_object_blank_close (metadata); /* GeoPoint */
@@ -617,8 +606,7 @@ extract_png (const gchar          *uri,
 	tracker_sparql_builder_object (metadata, "nfo:Image");
 	tracker_sparql_builder_object (metadata, "nmm:Photo");
 
-	read_metadata (preupdate, metadata, png_ptr, info_ptr, uri);
-	read_metadata (preupdate, metadata, png_ptr, end_ptr, uri);
+	read_metadata (preupdate, metadata, png_ptr, info_ptr, end_ptr, uri);
 
 	tracker_sparql_builder_predicate (metadata, "nfo:width");
 	tracker_sparql_builder_object_int64 (metadata, width);
