@@ -2,16 +2,16 @@
  * Copyright (C) 2008, Nokia <ivan.frade@nokia.com>
  *
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public
+ * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 2.1 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public
+ * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
  * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
  * Boston, MA  02110-1301, USA.
@@ -27,12 +27,12 @@
 
 #include <gio/gunixoutputstream.h>
 #include <gio/gunixinputstream.h>
+#include <gio/gunixfdlist.h>
 
 #include <libtracker-common/tracker-common.h>
 
 #include <libtracker-extract/tracker-extract.h>
 
-#include "tracker-dbus.h"
 #include "tracker-extract.h"
 #include "tracker-main.h"
 #include "tracker-marshal.h"
@@ -48,6 +48,26 @@
 #define UNKNOWN_METHOD_MESSAGE "Method \"%s\" with signature \"%s\" on " \
                                "interface \"%s\" doesn't exist, expected \"%s\""
 
+static const gchar introspection_xml[] =
+  "<node>"
+  "  <interface name='org.freedesktop.Tracker1.Extract'>"
+  "    <method name='GetPid'>"
+  "      <arg type='i' name='value' direction='out' />"
+  "    </method>"
+  "    <method name='GetMetadata'>"
+  "      <arg type='s' name='uri' direction='in' />"
+  "      <arg type='s' name='mime' direction='in' />"
+  "      <arg type='s' name='preupdate' direction='out' />"
+  "      <arg type='s' name='embedded' direction='out' />"
+  "    </method>"
+  "    <method name='GetMetadataFast'>"
+  "      <arg type='s' name='uri' direction='in' />"
+  "      <arg type='s' name='mime' direction='in' />"
+  "      <arg type='h' name='fd' direction='in' />"
+  "    </method>"
+  "  </interface>"
+  "</node>";
+
 #define TRACKER_EXTRACT_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), TRACKER_TYPE_EXTRACT, TrackerExtractPrivate))
 
 extern gboolean debug;
@@ -58,6 +78,9 @@ typedef struct {
 	gboolean disable_shutdown;
 	gboolean force_internal_extractors;
 	gboolean disable_summary_on_finalize;
+	GDBusConnection *d_connection;
+	GDBusNodeInfo *introspection_data;
+	guint registration_id;
 } TrackerExtractPrivate;
 
 typedef struct {
@@ -214,12 +237,12 @@ load_modules (const gchar  *force_module,
 	gboolean success;
 	const gchar *extractors_dir;
 
-        extractors_dir = g_getenv ("TRACKER_EXTRACTORS_DIR");
-        if (G_LIKELY (extractors_dir == NULL)) {
-                extractors_dir = TRACKER_EXTRACTORS_DIR;
-        } else {
-                g_message ("Extractor modules directory is '%s' (set in env)", extractors_dir);
-        }
+	extractors_dir = g_getenv ("TRACKER_EXTRACTORS_DIR");
+	if (G_LIKELY (extractors_dir == NULL)) {
+		extractors_dir = TRACKER_EXTRACTORS_DIR;
+	} else {
+		g_message ("Extractor modules directory is '%s' (set in env)", extractors_dir);
+	}
 
 	dir = g_dir_open (extractors_dir, 0, &error);
 
@@ -368,12 +391,12 @@ tracker_extract_new (gboolean     disable_shutdown,
 
 static gboolean
 get_file_metadata (TrackerExtract         *extract,
-                   guint                   request_id,
-                   DBusGMethodInvocation  *context,
+                   TrackerDBusRequest     *request,
+                   GDBusMethodInvocation  *invocation,
                    const gchar            *uri,
                    const gchar            *mime,
-		   TrackerSparqlBuilder  **preupdate_out,
-		   TrackerSparqlBuilder  **statements_out)
+                   TrackerSparqlBuilder  **preupdate_out,
+                   TrackerSparqlBuilder  **statements_out)
 {
 	TrackerExtractPrivate *priv;
 	TrackerSparqlBuilder *statements, *preupdate;
@@ -393,7 +416,7 @@ get_file_metadata (TrackerExtract         *extract,
 
 #ifdef HAVE_LIBSTREAMANALYZER
 	if (!priv->force_internal_extractors) {
-		tracker_dbus_request_comment (request_id, context,
+		tracker_dbus_request_comment (request,
 		                              "  Extracting with libstreamanalyzer...");
 
 		tracker_topanalyzer_extract (uri, statements, &content_type);
@@ -407,7 +430,7 @@ get_file_metadata (TrackerExtract         *extract,
 			return TRUE;
 		}
 	} else {
-		tracker_dbus_request_comment (request_id, context,
+		tracker_dbus_request_comment (request,
 		                              "  Extracting with internal extractors ONLY...");
 	}
 #endif /* HAVE_LIBSTREAMANALYZER */
@@ -445,8 +468,7 @@ get_file_metadata (TrackerExtract         *extract,
 		                          &error);
 
 		if (error || !info) {
-			tracker_dbus_request_comment (request_id,
-			                              context,
+			tracker_dbus_request_comment (request,
 			                              "  Could not create GFileInfo for file size check, %s",
 			                              error ? error->message : "no error given");
 			g_error_free (error);
@@ -464,8 +486,7 @@ get_file_metadata (TrackerExtract         *extract,
 
 		mime_used = g_strdup (g_file_info_get_content_type (info));
 
-		tracker_dbus_request_comment (request_id,
-		                              context,
+		tracker_dbus_request_comment (request,
 		                              "  Guessing mime type as '%s' for uri:'%s'",
 		                              mime_used,
 		                              uri);
@@ -500,8 +521,7 @@ get_file_metadata (TrackerExtract         *extract,
 			if (g_pattern_match (mdata->pattern, length, mime_used, reversed)) {
 				gint items;
 
-				tracker_dbus_request_comment (request_id,
-				                              context,
+				tracker_dbus_request_comment (request,
 				                              "  Extracting with module:'%s'",
 				                              g_module_name ((GModule*) mdata->module));
 
@@ -509,8 +529,7 @@ get_file_metadata (TrackerExtract         *extract,
 
 				items = tracker_sparql_builder_get_length (statements);
 
-				tracker_dbus_request_comment (request_id,
-				                              context,
+				tracker_dbus_request_comment (request,
 				                              "  Found %d metadata items",
 				                              items);
 
@@ -542,8 +561,7 @@ get_file_metadata (TrackerExtract         *extract,
 			if (g_pattern_match (mdata->pattern, length, mime_used, reversed)) {
 				gint items;
 
-				tracker_dbus_request_comment (request_id,
-				                              context,
+				tracker_dbus_request_comment (request,
 				                              "  Extracting with module:'%s'",
 				                              g_module_name ((GModule*) mdata->module));
 
@@ -551,8 +569,7 @@ get_file_metadata (TrackerExtract         *extract,
 
 				items = tracker_sparql_builder_get_length (statements);
 
-				tracker_dbus_request_comment (request_id,
-				                              context,
+				tracker_dbus_request_comment (request,
 				                              "  Found %d metadata items",
 				                              items);
 
@@ -575,8 +592,7 @@ get_file_metadata (TrackerExtract         *extract,
 			}
 		}
 
-		tracker_dbus_request_comment (request_id,
-		                              context,
+		tracker_dbus_request_comment (request,
 		                              "  Could not find any extractors to handle metadata type "
 		                              "(mime: %s)",
 		                              mime_used);
@@ -584,8 +600,7 @@ get_file_metadata (TrackerExtract         *extract,
 		g_free (mime_used);
 		g_free (reversed);
 	} else {
-		tracker_dbus_request_comment (request_id,
-		                              context,
+		tracker_dbus_request_comment (request,
 		                              "  No mime available, not extracting data");
 	}
 
@@ -604,27 +619,24 @@ tracker_extract_get_metadata_by_cmdline (TrackerExtract *object,
                                          const gchar    *uri,
                                          const gchar    *mime)
 {
-	guint request_id;
+	TrackerDBusRequest *request;
 	TrackerSparqlBuilder *statements, *preupdate;
 	TrackerExtractPrivate *priv;
 
-	request_id = tracker_dbus_get_next_request_id ();
-
 	priv = TRACKER_EXTRACT_GET_PRIVATE (object);
 	priv->disable_summary_on_finalize = TRUE;
 
 	g_return_if_fail (uri != NULL);
 
-	tracker_dbus_request_new (request_id,
-	                          NULL,
-	                          "%s(uri:'%s', mime:%s)",
-	                          __FUNCTION__,
-	                          uri,
-	                          mime);
+	request = tracker_dbus_request_begin (NULL,
+	                                      "%s(uri:'%s', mime:%s)",
+	                                      __FUNCTION__,
+	                                      uri,
+	                                      mime);
 
 	/* NOTE: Don't reset the timeout to shutdown here */
 
-	if (get_file_metadata (object, request_id,
+	if (get_file_metadata (object, request,
 			       NULL, uri, mime,
 			       &preupdate, &statements)) {
 		const gchar *preupdate_str, *statements_str;
@@ -639,68 +651,63 @@ tracker_extract_get_metadata_by_cmdline (TrackerExtract *object,
 			preupdate_str = tracker_sparql_builder_get_result (preupdate);
 		}
 
-		tracker_dbus_request_info (request_id, NULL, "%s",
-					   preupdate_str ? preupdate_str : "");
-		tracker_dbus_request_info (request_id, NULL, "%s",
-					   statements_str ? statements_str : "");
+		tracker_dbus_request_info (request, "%s",
+		                           preupdate_str ? preupdate_str : "");
+		tracker_dbus_request_info (request, "%s",
+		                           statements_str ? statements_str : "");
 
 		g_object_unref (statements);
 		g_object_unref (preupdate);
 	}
 
-	tracker_dbus_request_success (request_id, NULL);
+	tracker_dbus_request_end (request, NULL);
 }
 
-void
-tracker_extract_get_pid (TrackerExtract         *object,
-                         DBusGMethodInvocation  *context,
-                         GError                **error)
+static void
+handle_method_call_get_pid (TrackerExtract        *object,
+                            GDBusMethodInvocation *invocation,
+                            GVariant              *parameters)
 {
-	guint request_id;
+	TrackerDBusRequest *request;
 	pid_t value;
 
-	request_id = tracker_dbus_get_next_request_id ();
-
-	tracker_dbus_request_new (request_id,
-	                          context,
-	                          "%s()",
-	                          __FUNCTION__);
+	request = tracker_g_dbus_request_begin (invocation,
+	                                        "%s()",
+	                                        __FUNCTION__);
 
 	value = getpid ();
-	tracker_dbus_request_debug (request_id,
-	                            context,
+	tracker_dbus_request_debug (request,
 	                            "PID is %d",
 	                            value);
 
-	tracker_dbus_request_success (request_id, context);
-	dbus_g_method_return (context, value);
+	tracker_dbus_request_end (request, NULL);
+
+	g_dbus_method_invocation_return_value (invocation,
+	                                       g_variant_new ("(i)", (gint) value));
 }
 
-void
-tracker_extract_get_metadata (TrackerExtract         *object,
-                              const gchar            *uri,
-                              const gchar            *mime,
-                              DBusGMethodInvocation  *context,
-                              GError                **error)
+static void
+handle_method_call_get_metadata (TrackerExtract        *object,
+                                 GDBusMethodInvocation *invocation,
+                                 GVariant              *parameters)
 {
-	guint request_id;
+	TrackerDBusRequest *request;
 	TrackerExtractPrivate *priv;
 	TrackerSparqlBuilder *sparql, *preupdate;
 	gboolean extracted = FALSE;
+	const gchar *uri = NULL, *mime = NULL;
 
-	request_id = tracker_dbus_get_next_request_id ();
+	g_variant_get (parameters, "(&s&s)", &uri, &mime);
 
-	tracker_dbus_async_return_if_fail (uri != NULL, context);
+	tracker_gdbus_async_return_if_fail (uri != NULL, invocation);
 
-	tracker_dbus_request_new (request_id,
-	                          context,
-	                          "%s(uri:'%s', mime:%s)",
-	                          __FUNCTION__,
-	                          uri,
-	                          mime);
+	request = tracker_g_dbus_request_begin (invocation,
+	                                        "%s(uri:'%s', mime:%s)",
+	                                        __FUNCTION__,
+	                                        uri,
+	                                        mime);
 
-	tracker_dbus_request_debug (request_id,
-	                            context,
+	tracker_dbus_request_debug (request,
 	                            "  Resetting shutdown timeout");
 
 	priv = TRACKER_EXTRACT_GET_PRIVATE (object);
@@ -710,10 +717,16 @@ tracker_extract_get_metadata (TrackerExtract         *object,
 		alarm (MAX_EXTRACT_TIME);
 	}
 
-	extracted = get_file_metadata (object, request_id, context, uri, mime, &preupdate, &sparql);
+	extracted = get_file_metadata (object,
+	                               request,
+	                               invocation,
+	                               uri,
+	                               mime,
+	                               &preupdate,
+	                               &sparql);
 
 	if (extracted) {
-		tracker_dbus_request_success (request_id, context);
+		tracker_dbus_request_end (request, NULL);
 
 		if (tracker_sparql_builder_get_length (sparql) > 0) {
 			const gchar *preupdate_str = NULL;
@@ -722,25 +735,26 @@ tracker_extract_get_metadata (TrackerExtract         *object,
 				preupdate_str = tracker_sparql_builder_get_result (preupdate);
 			}
 
-			dbus_g_method_return (context,
-			                      preupdate_str ? preupdate_str : "",
-			                      tracker_sparql_builder_get_result (sparql));
+			g_dbus_method_invocation_return_value (invocation,
+			                                       g_variant_new ("(ss)",
+			                                                      preupdate_str ? preupdate_str : "",
+			                                                      tracker_sparql_builder_get_result (sparql)));
 		} else {
-			dbus_g_method_return (context, "", "");
+			g_dbus_method_invocation_return_value (invocation,
+			                                       g_variant_new ("(ss)", "", ""));
 		}
 
 		g_object_unref (sparql);
 		g_object_unref (preupdate);
 	} else {
-		GError *actual_error = NULL;
+		GError *actual_error;
 
-		tracker_dbus_request_failed (request_id,
-		                             context,
-		                             &actual_error,
-		                             "Could not get any metadata for uri:'%s' and mime:'%s'",
-		                             uri,
-		                             mime);
-		dbus_g_method_return_error (context, actual_error);
+		actual_error = g_error_new (TRACKER_DBUS_ERROR, 0,
+		                            "Could not get any metadata for uri:'%s' and mime:'%s'",
+		                            uri,
+		                            mime);
+		tracker_dbus_request_end (request, actual_error);
+		g_dbus_method_invocation_return_gerror (invocation, actual_error);
 		g_error_free (actual_error);
 	}
 
@@ -751,93 +765,48 @@ tracker_extract_get_metadata (TrackerExtract         *object,
 }
 
 static void
-get_metadata_fast (TrackerExtract *object,
-                   DBusConnection *connection,
-                   DBusMessage    *message)
+handle_method_call_get_metadata_fast (TrackerExtract        *object,
+                                      GDBusMethodInvocation *invocation,
+                                      GVariant              *parameters)
 {
-	guint request_id;
-	const gchar *expected_signature;
+	TrackerDBusRequest *request;
 	TrackerExtractPrivate *priv;
-	DBusError dbus_error;
-	DBusMessage *reply;
+	GDBusMessage *reply;
 	const gchar *uri, *mime;
-	int fd;
+	int fd, index_fd;
 	GOutputStream *unix_output_stream;
 	GOutputStream *buffered_output_stream;
 	GDataOutputStream *data_output_stream;
 	GError *error = NULL;
 	TrackerSparqlBuilder *sparql, *preupdate;
 	gboolean extracted = FALSE;
+	GDBusMessage *method_message;
+	GDBusConnection *connection;
+	GUnixFDList *fd_list;
 
-	request_id = tracker_dbus_get_next_request_id ();
+	connection = g_dbus_method_invocation_get_connection (invocation);
+	method_message = g_dbus_method_invocation_get_message (invocation);
 
-	expected_signature = DBUS_TYPE_STRING_AS_STRING DBUS_TYPE_STRING_AS_STRING DBUS_TYPE_UNIX_FD_AS_STRING;
+	g_variant_get (parameters, "(&s&sh)", &uri, &mime, &index_fd);
 
-	if (g_strcmp0 (dbus_message_get_signature (message), expected_signature)) {
-		gchar *error_message = g_strdup_printf (UNKNOWN_METHOD_MESSAGE,
-		                                        "GetMetadataFast",
-		                                        dbus_message_get_signature (message),
-		                                        dbus_message_get_interface (message),
-		                                        expected_signature);
-		tracker_dbus_request_new (request_id,
-		                          NULL,
-		                          "%s()",
-		                          __FUNCTION__);
+	fd_list = g_dbus_message_get_unix_fd_list (method_message);
 
-		reply = dbus_message_new_error (message,
-		                                DBUS_ERROR_UNKNOWN_METHOD,
-		                                error_message);
-		dbus_connection_send (connection, reply, NULL);
+	request = tracker_g_dbus_request_begin (invocation,
+	                                        "%s(uri:'%s', mime:%s)",
+	                                        __FUNCTION__,
+	                                        uri,
+	                                        mime);
 
-		tracker_dbus_request_failed (request_id,
-		                             NULL,
-		                             NULL,
-		                             error_message);
-
-		dbus_message_unref (reply);
-		g_free (error_message);
-
-		return;
+	if ((fd = g_unix_fd_list_get (fd_list, index_fd, &error)) == -1) {
+		tracker_dbus_request_end (request, error);
+		reply = g_dbus_message_new_method_error_literal (method_message,
+		                                                 TRACKER_EXTRACT_SERVICE ".GetMetadataFastError",
+		                                                 error->message);
+		g_error_free (error);
+		goto bail_out;
 	}
 
-	dbus_error_init (&dbus_error);
-
-	dbus_message_get_args (message,
-	                       &dbus_error,
-	                       DBUS_TYPE_STRING, &uri,
-	                       DBUS_TYPE_STRING, &mime,
-	                       DBUS_TYPE_UNIX_FD, &fd,
-	                       DBUS_TYPE_INVALID);
-
-	if (dbus_error_is_set (&dbus_error)) {
-		tracker_dbus_request_new (request_id,
-		                          NULL,
-		                          "%s()",
-		                          __FUNCTION__);
-
-		reply = dbus_message_new_error (message, dbus_error.name, dbus_error.message);
-		dbus_connection_send (connection, reply, NULL);
-
-		tracker_dbus_request_failed (request_id,
-		                             NULL,
-		                             NULL,
-		                             dbus_error.message);
-
-		dbus_message_unref (reply);
-		dbus_error_free (&dbus_error);
-
-		return;
-	}
-
-	tracker_dbus_request_new (request_id,
-	                          NULL,
-	                          "%s(uri:'%s', mime:%s)",
-	                          __FUNCTION__,
-	                          uri,
-	                          mime);
-
-	tracker_dbus_request_debug (request_id,
-	                            NULL,
+	tracker_dbus_request_debug (request,
 	                            "  Resetting shutdown timeout");
 
 	priv = TRACKER_EXTRACT_GET_PRIVATE (object);
@@ -847,7 +816,7 @@ get_metadata_fast (TrackerExtract *object,
 		alarm (MAX_EXTRACT_TIME);
 	}
 
-	extracted = get_file_metadata (object, request_id, NULL, uri, mime, &preupdate, &sparql);
+	extracted = get_file_metadata (object, request, NULL, uri, mime, &preupdate, &sparql);
 
 	if (extracted) {
 		unix_output_stream = g_unix_output_stream_new (fd, TRUE);
@@ -898,70 +867,184 @@ get_metadata_fast (TrackerExtract *object,
 		g_object_unref (unix_output_stream);
 
 		if (error) {
-			tracker_dbus_request_failed (request_id,
-			                             NULL,
-			                             &error,
-			                             NULL);
-			reply = dbus_message_new_error (message,
-			                                TRACKER_EXTRACT_SERVICE ".GetMetadataFastError",
-			                                error->message);
-			dbus_connection_send (connection, reply, NULL);
-			dbus_message_unref (reply);
+			tracker_dbus_request_end (request, error);
+			reply = g_dbus_message_new_method_error_literal (method_message,
+			                                                 TRACKER_EXTRACT_SERVICE ".GetMetadataFastError",
+			                                                 error->message);
 			g_error_free (error);
 		} else {
-			tracker_dbus_request_success (request_id, NULL);
-			reply = dbus_message_new_method_return (message);
-			dbus_connection_send (connection, reply, NULL);
-			dbus_message_unref (reply);
+			tracker_dbus_request_end (request, NULL);
+			reply = g_dbus_message_new_method_reply (method_message);
 		}
 	} else {
-		gchar *error_message = g_strdup_printf ("Could not get any metadata for uri:'%s' and mime:'%s'", uri, mime);
-		tracker_dbus_request_failed (request_id,
-		                             NULL,
-		                             NULL,
-		                             error_message);
-		reply = dbus_message_new_error (message,
-		                                TRACKER_EXTRACT_SERVICE ".GetMetadataFastError",
-		                                error_message);
+		error = g_error_new (TRACKER_DBUS_ERROR, 0,
+		                     "Could not get any metadata for uri:'%s' and mime:'%s'", uri, mime);
+		tracker_dbus_request_end (request, error);
+		reply = g_dbus_message_new_method_error_literal (method_message,
+		                                                 TRACKER_EXTRACT_SERVICE ".GetMetadataFastError",
+		                                                 error->message);
+		g_error_free (error);
 		close (fd);
-		dbus_connection_send (connection, reply, NULL);
-		dbus_message_unref (reply);
-		g_free (error_message);
 	}
 
+bail_out:
+
+	g_dbus_connection_send_message (connection, reply,
+	                                G_DBUS_SEND_MESSAGE_FLAGS_NONE,
+	                                NULL, NULL);
+
+	g_object_unref (fd_list);
+	g_object_unref (reply);
+
 	if (!priv->disable_shutdown) {
 		/* Unset alarm so the extractor doesn't die when it's idle */
 		alarm (0);
 	}
 }
 
-DBusHandlerResult
-tracker_extract_connection_filter (DBusConnection *connection,
-                                   DBusMessage    *message,
-                                   void           *user_data)
+static void
+handle_method_call (GDBusConnection       *connection,
+                    const gchar           *sender,
+                    const gchar           *object_path,
+                    const gchar           *interface_name,
+                    const gchar           *method_name,
+                    GVariant              *parameters,
+                    GDBusMethodInvocation *invocation,
+                    gpointer               user_data)
 {
-	TrackerExtract *extract;
+	TrackerExtract *extract = user_data;
 
-	g_return_val_if_fail (connection != NULL, DBUS_HANDLER_RESULT_NOT_YET_HANDLED);
-	g_return_val_if_fail (message != NULL, DBUS_HANDLER_RESULT_NOT_YET_HANDLED);
+	if (g_strcmp0 (method_name, "GetPid") == 0) {
+		handle_method_call_get_pid (extract, invocation, parameters);
+	} else
+	if (g_strcmp0 (method_name, "GetMetadataFast") == 0) {
+		handle_method_call_get_metadata_fast (extract, invocation, parameters);
+	} else
+	if (g_strcmp0 (method_name, "GetMetadata") == 0) {
+		handle_method_call_get_metadata (extract, invocation, parameters);
+	} else {
+		g_assert_not_reached ();
+	}
+}
 
-	if (g_strcmp0 (TRACKER_EXTRACT_PATH, dbus_message_get_path (message))) {
-		return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
+static GVariant *
+handle_get_property (GDBusConnection  *connection,
+                     const gchar      *sender,
+                     const gchar      *object_path,
+                     const gchar      *interface_name,
+                     const gchar      *property_name,
+                     GError          **error,
+                     gpointer          user_data)
+{
+	g_assert_not_reached ();
+	return NULL;
+}
+
+static gboolean
+handle_set_property (GDBusConnection  *connection,
+                     const gchar      *sender,
+                     const gchar      *object_path,
+                     const gchar      *interface_name,
+                     const gchar      *property_name,
+                     GVariant         *value,
+                     GError          **error,
+                     gpointer          user_data)
+{
+	g_assert_not_reached ();
+	return TRUE;
+}
+
+void
+tracker_extract_dbus_start (TrackerExtract *extract)
+{
+	TrackerExtractPrivate *priv;
+	GVariant *reply;
+	guint32 rval;
+	GError *error = NULL;
+	GDBusInterfaceVTable interface_vtable = {
+		handle_method_call,
+		handle_get_property,
+		handle_set_property
+	};
+
+	priv = TRACKER_EXTRACT_GET_PRIVATE (extract);
+
+	priv->d_connection = g_bus_get_sync (G_BUS_TYPE_SESSION, NULL, &error);
+
+	if (!priv->d_connection) {
+		g_critical ("Could not connect to the D-Bus session bus, %s",
+		            error ? error->message : "no error given.");
+		g_clear_error (&error);
+		return;
+	}
+
+	priv->introspection_data = g_dbus_node_info_new_for_xml (introspection_xml, NULL);
+
+	g_message ("Registering D-Bus object...");
+	g_message ("  Path:'" TRACKER_EXTRACT_PATH "'");
+	g_message ("  Object Type:'%s'", G_OBJECT_TYPE_NAME (extract));
+
+	priv->registration_id =
+		g_dbus_connection_register_object (priv->d_connection,
+		                                   TRACKER_EXTRACT_PATH,
+		                                   priv->introspection_data->interfaces[0],
+		                                   &interface_vtable,
+		                                   extract,
+		                                   NULL,
+		                                   &error);
+
+	if (error) {
+		g_critical ("Could not register the D-Bus object "TRACKER_EXTRACT_PATH", %s",
+		            error ? error->message : "no error given.");
+		g_clear_error (&error);
+		return;
+	}
+
+	reply = g_dbus_connection_call_sync (priv->d_connection,
+	                                     "org.freedesktop.DBus",
+	                                     "/org/freedesktop/DBus",
+	                                     "org.freedesktop.DBus",
+	                                     "RequestName",
+	                                     g_variant_new ("(su)", TRACKER_EXTRACT_SERVICE, 0x4 /* DBUS_NAME_FLAG_DO_NOT_QUEUE */),
+	                                     G_VARIANT_TYPE ("(u)"),
+	                                     0, -1, NULL, &error);
+
+	if (error) {
+		g_critical ("Could not acquire name:'%s', %s",
+		            TRACKER_EXTRACT_SERVICE,
+		            error->message);
+		g_clear_error (&error);
+		return;
+	}
+
+	g_variant_get (reply, "(u)", &rval);
+	g_variant_unref (reply);
+
+	if (rval != 1 /* DBUS_REQUEST_NAME_REPLY_PRIMARY_OWNER */) {
+		g_critical ("D-Bus service name:'%s' is already taken, "
+		            "perhaps the application is already running?",
+		            TRACKER_EXTRACT_SERVICE);
+		return;
+	}
+}
+
+void
+tracker_extract_dbus_stop (TrackerExtract *extract)
+{
+	TrackerExtractPrivate *priv;
+
+	priv = TRACKER_EXTRACT_GET_PRIVATE (extract);
+
+	if (priv->registration_id != 0) {
+		g_dbus_connection_unregister_object (priv->d_connection,
+		                                     priv->registration_id);
+	}
+
+	if (priv->introspection_data) {
+		g_dbus_node_info_unref (priv->introspection_data);
+	}
+
+	if (priv->d_connection) {
+		g_object_unref (priv->d_connection);
 	}
-
-	if (g_strcmp0 (TRACKER_EXTRACT_INTERFACE, dbus_message_get_interface (message))) {
-		return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
-	}
-
-	/* Only check if the user_data is our TrackerExtract AFTER having checked that
-	 * the message matches expected path and interface. */
-	extract = user_data;
-	g_return_val_if_fail (TRACKER_IS_EXTRACT (extract), DBUS_HANDLER_RESULT_NOT_YET_HANDLED);
-
-	if (!g_strcmp0 ("GetMetadataFast", dbus_message_get_member (message))) {
-		get_metadata_fast (extract, connection, message);
-		return DBUS_HANDLER_RESULT_HANDLED;
-	}
-
-	return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
 }
