@@ -9,10 +9,16 @@ import dbus
 from dbus.mainloop.glib import DBusGMainLoop
 import time
 
+import options
+#from common.utils.options import get_start_timeout
+
 # Don't use /tmp (not enough space there)
 
+# Add this after fixing the backup/restore and ontology changes tests
+#"G_DEBUG" : "fatal_criticals",
+
 TEST_ENV_VARS =  { "XDG_DATA_HOME" : os.path.join (cfg.TEST_TMP_DIR, "xdg-data-home"),
-                   "XDG_CACHE_HOME": os.path.join (cfg.TEST_TMP_DIR, "xdg-cache-home")} 
+                   "XDG_CACHE_HOME": os.path.join (cfg.TEST_TMP_DIR, "xdg-cache-home")}
 EXTRA_DIRS = [os.path.join (cfg.TEST_TMP_DIR, "xdg-data-home", "tracker"),
               os.path.join (cfg.TEST_TMP_DIR, "xdg-cache-home", "tracker")]
 
@@ -28,7 +34,8 @@ class TrackerStoreLifeCycle ():
 
     def __init__ (self):
         self.timeout_id = 0
-
+        self.available = False
+        
     def start (self):
         """
         call this method to start and instance of tracker-store. It will return when the store is ready
@@ -48,7 +55,7 @@ class TrackerStoreLifeCycle ():
                                                          path="/org/freedesktop/DBus",
                                                          dbus_interface="org.freedesktop.DBus")
         self.store_proc = self.__start_tracker_store ()
-        
+
         # It should step out of this loop when the miner is visible in DBus
         self.loop.run ()
 
@@ -59,12 +66,13 @@ class TrackerStoreLifeCycle ():
         self.status_iface.Wait ()
         print "[store] ready."
 
-        
+
     def stop (self):
         self.__stop_tracker_store ()
-        # It should step out of this loop when the miner disappear from the bus
-        self.timeout_id = glib.timeout_add_seconds (REASONABLE_TIMEOUT, self.__timeout_on_idle)
-        self.loop.run ()
+        if self.available:
+            # It should step out of this loop when the miner disappear from the bus
+            self.timeout_id = glib.timeout_add_seconds (REASONABLE_TIMEOUT, self.__timeout_on_idle)
+            self.loop.run ()
 
         print "[store] stop."
         # Disconnect the signals of the next start we get duplicated messages
@@ -87,8 +95,10 @@ class TrackerStoreLifeCycle ():
         if name == cfg.TRACKER_BUSNAME:
             if old_owner == '' and new_owner != '':
                 print "[store] appears in the bus"
+                self.available = True
             elif old_owner != ''  and new_owner == '':
                 print "[store] disappears from the bus"
+                self.available = False
             else:
                 print "[store] name change %s -> %s" % (old_owner, new_owner)
             self.loop.quit ()
@@ -98,13 +108,22 @@ class TrackerStoreLifeCycle ():
         tracker = [tracker_binary]
         # The env variables can be passed as parameters!
         FNULL = open('/dev/null', 'w')
-        return subprocess.Popen (tracker, stdout=FNULL, stderr=FNULL)
+        if options.is_manual_start ():
+            print "Start tracker-store manually"
+        else:
+            return subprocess.Popen (tracker, stdout=FNULL, stderr=FNULL)
 
     def __stop_tracker_store (self):
         #control_binary = os.path.join (cfg.BINDIR, "tracker-control")
         #FNULL = open('/dev/null', 'w')
         #subprocess.call ([control_binary, "-t"], stdout=FNULL)
-        self.store_proc.terminate ()
+        if options.is_manual_start ():
+            if self.available:
+                print "Kill tracker-store manually"
+                # Quit when disappearing from the bus
+                self.loop.run ()
+        else:
+            self.store_proc.terminate ()
 
 
 class TrackerMinerFsLifeCycle():
@@ -113,7 +132,8 @@ class TrackerMinerFsLifeCycle():
     """
     def __init__ (self):
         self.timeout_id = 0
-    
+        self.available = False
+
     def start (self):
         """
         call this method to start and instance of miner-fs. It will return when the miner is 'Idle'
@@ -134,7 +154,7 @@ class TrackerMinerFsLifeCycle():
                                                          path="/org/freedesktop/DBus",
                                                          dbus_interface="org.freedesktop.DBus")
         self.__start_tracker_miner_fs ()
-        
+
         # It should step out of this loop when the miner is visible in DBus
         self.loop.run ()
 
@@ -146,21 +166,23 @@ class TrackerMinerFsLifeCycle():
         self.timeout_id = glib.timeout_add_seconds (REASONABLE_TIMEOUT, self.__timeout_on_idle)
         self.loop.run ()
 
-        
+
     def stop (self):
         self.__stop_tracker_miner_fs ()
         # It should step out of this loop when the miner disappear from the bus
-        self.timeout_id = glib.timeout_add_seconds (REASONABLE_TIMEOUT, self.__timeout_on_idle)
-        self.loop.run ()
+        if (self.available):
+            self.timeout_id = glib.timeout_add_seconds (REASONABLE_TIMEOUT, self.__timeout_on_idle)
+            self.loop.run ()
 
         # Disconnect the signals of the next start we get duplicated messages
         self.bus._clean_up_signal_match (self.name_owner_match)
         self.bus._clean_up_signal_match (self.status_match)
+        print "[miner-fs] stop."
 
 
     def wait_for_idle (self, timeout=REASONABLE_TIMEOUT):
         # The signal is already connected
-        print "[miner-fs] waiting for Idle"
+        print "\n[miner-fs] waiting for Idle"
         self.timeout_id = glib.timeout_add_seconds (timeout, self.__timeout_on_idle)
         self.loop.run ()
 
@@ -191,12 +213,19 @@ class TrackerMinerFsLifeCycle():
     def __start_tracker_miner_fs (self):
         miner_fs_binary = os.path.join (cfg.EXEC_PREFIX, "tracker-miner-fs")
         FNULL = open ('/dev/null', 'w')
-        return subprocess.Popen ([miner_fs_binary], stdout=FNULL, stderr=FNULL)
+        if options.is_manual_start ():
+            print "Start tracker-miner-fs manually"
+        else:
+            return subprocess.Popen ([miner_fs_binary], stdout=FNULL, stderr=FNULL)
 
     def __stop_tracker_miner_fs (self):
         control_binary = os.path.join (cfg.BINDIR, "tracker-control")
         FNULL = open('/dev/null', 'w')
-        subprocess.call ([control_binary, "-t"], stdout=FNULL)
+        if options.is_manual_start ():
+            print "Kill miner manually"
+            self.loop.run ()
+        else:
+            subprocess.call ([control_binary, "-t"], stdout=FNULL)
 
 
 class TrackerWritebackLifeCycle():
@@ -205,7 +234,7 @@ class TrackerWritebackLifeCycle():
     """
     def __init__ (self):
         self.timeout_id = 0
-    
+
     def start (self):
         """
         call this method to start and instance of writeback.
@@ -230,19 +259,24 @@ class TrackerWritebackLifeCycle():
         # It should step out of this loop when the writeback is visible in DBus
         self.timeout_id = glib.timeout_add_seconds (REASONABLE_TIMEOUT, self.__timeout_on_idle)
         self.loop.run ()
-        
+
     def stop (self):
-        assert self.process 
+        assert self.process
         self.process.kill ()
 
     def __name_owner_changed_cb (self, name, old_owner, new_owner):
         if name == cfg.WRITEBACK_BUSNAME:
-            print "Writeback changed named '%s' -> '%s'" % (old_owner, new_owner)
+            if old_owner == '' and new_owner != '':
+                print "[writeback] appears in the bus"
+            elif old_owner != ''  and new_owner == '':
+                print "[writeback] disappears from the bus"
+            else:
+                print "[writeback] name change %s -> %s" % (old_owner, new_owner)
             if (self.timeout_id != 0):
                 glib.source_remove (self.timeout_id)
                 self.timeout_id = 0
             self.loop.quit ()
-            
+
     def __timeout_on_idle (self):
         print "Timeout... asumming idle"
         self.loop.quit ()
@@ -253,7 +287,10 @@ class TrackerWritebackLifeCycle():
         writeback = [writeback_binary]
         # The env variables can be passed as parameters!
         FNULL = open('/dev/null', 'w')
-        self.process = subprocess.Popen (writeback, stdout=FNULL, stderr=FNULL)
+        if options.is_manual_start ():
+            print "Start tracker-writeback manually"
+        else:
+            self.process = subprocess.Popen (writeback, stdout=FNULL, stderr=FNULL)
 
 
 class TrackerSystemAbstraction:
@@ -269,7 +306,7 @@ class TrackerSystemAbstraction:
 
         for directory in EXTRA_DIRS:
             self.__recreate_directory (directory)
-            
+
         if confdir :
             self.__recreate_directory (XDG_CONFIG_HOME_DIR)
             shutil.copytree (os.path.join (confdir, "tracker"),
@@ -289,7 +326,7 @@ class TrackerSystemAbstraction:
         for var, directory in TEST_ENV_VARS.iteritems ():
             if os.environ.has_key (var):
                 del os.environ [var]
-                
+
         if (os.environ.has_key ("XDG_CONFIG_HOME")):
             del os.environ ["XDG_CONFIG_HOME"]
 
@@ -303,7 +340,7 @@ class TrackerSystemAbstraction:
         """
         self.__stop_tracker_processes ()
         self.set_up_environment (confdir, ontodir)
-        
+
         self.store = TrackerStoreLifeCycle ()
         self.store.start ()
 
@@ -349,7 +386,7 @@ class TrackerSystemAbstraction:
         db_location = os.path.join (TEST_ENV_VARS ['XDG_CACHE_HOME'], "tracker")
         shutil.rmtree (db_location)
         os.mkdir (db_location)
-        
+
 
     def tracker_store_testing_stop (self):
         """
@@ -368,6 +405,10 @@ class TrackerSystemAbstraction:
         self.__stop_tracker_processes ()
         self.set_up_environment (confdir, None)
 
+        # Start also the store. DBus autoactivation ignores the env variables.
+        self.store = TrackerStoreLifeCycle ()
+        self.store.start ()
+
         self.miner_fs = TrackerMinerFsLifeCycle ()
         self.miner_fs.start ()
 
@@ -383,6 +424,8 @@ class TrackerSystemAbstraction:
         Stops the miner-fs and store running and unset all the XDG_*_HOME vars
         """
         self.miner_fs.stop ()
+        self.store.stop ()
+
         self.__stop_tracker_processes ()
         self.unset_up_environment ()
 
@@ -397,7 +440,15 @@ class TrackerSystemAbstraction:
         # Tracker write must have been started before
         self.writeback.stop ()
         self.tracker_miner_fs_testing_stop ()
-        
+
+    def tracker_all_testing_start (self, confdir=None):
+        # This will start all miner-fs, store and writeback
+        self.tracker_writeback_testing_start (confdir)
+
+    def tracker_all_testing_stop (self):
+        # This will stop all miner-fs, store and writeback
+        self.tracker_writeback_testing_stop ()
+
     #
     # Private API
     #
@@ -429,7 +480,7 @@ if __name__ == "__main__":
     print "   started, waiting 5 sec. to stop it"
     glib.timeout_add_seconds (5, destroy_the_world, a)
     gtk.main ()
-    
+
     print "-- Starting miner-fs --"
     b = TrackerMinerFsLifeCycle ()
     b.start ()
